#include "config.h"
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <sys/time.h>
#include "log.h"
#include "smtp.h"
#include "proxy.h"

#define SERVER_PORT 25

PROXY *proxy;


int _interrupted = 0;

void handle_interrupt(int signal)
{
  proxy_break(proxy);
}


int net_close(SMTP *c, void *data)
{
  log_put("Remote end closed connection.");

  proxy_del_smtp(proxy, c);
  smtp_del(c);
  return(0);
}


int block_recv(SMTP *c, TT *tt, void *data)
{
  log_put("You've got m@il!");

  tt_print_to_file(tt, stdout, TT_PRINT_ALLMAN, 0);
  return(0);
}


int net_connect(SMTP *c, void *data)
{
  log_put("Got connection.");

  proxy_add_smtp_in(proxy, c, 0, block_recv);
  proxy_add_smtp_close(proxy, c, 0, net_close);
  return(0);
}


int timeout_four(char *name, void *priv)
{
  printf("4"); fflush(stdout);
  return(0);
}


int timeout_one(char *name, void *priv)
{
  printf("1"); fflush(stdout);
  return(0);
}


int main()
{
  SMTP *c0;

  log_init("smtp-server", LOG_TYPE_GENERIC, LOG_TO_STDERR | LOG_TO_STDOUT | LOG_WITH_NAME, LOG_INFO);

  signal(SIGCLD, SIG_IGN);  /* Dispel undead (zombie daemon children) */
  signal(SIGINT, handle_interrupt);

  proxy = proxy_new();
  c0 = smtp_accepting_new(SERVER_PORT);
  if (!c0)
  {
    log_put("Couldn't open server socket.");
    exit(0);
  }

  proxy_add_smtp_connect(proxy, c0, 0, net_connect);

  log_put("Listening...");
  proxy_loop(proxy);

  log_put("Terminated.");
  proxy_del(proxy);
  return(0);
}
  
