/* Copyright (C) 1999-2000 Hans Petter K. Jansson
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * You can contact the library's author by sending e-mail to <hpj@styx.net>.
 */

#include "config.h"
#include "bmem.h"
#include "bt.h"
#include "bt_priv.h"
#include <stdlib.h>


BMEM *bt_bmem = NULL;


BT *bt_new()
{
  BT *bt;

  if (!bt_bmem) bt_bmem = bmem_new(sizeof(BTN));
  
  bt = malloc(sizeof(*bt));
  memset(bt, 0, sizeof(*bt));

  bt->bmem = bt_bmem;
  bt->head.right = &bt->z;
  bt->z.left = &bt->z; bt->z.right = &bt->z;
  
  return(bt);
}


BT *bt_new_without_bmem()
{
  BT *bt;

  bt = malloc(sizeof(*bt));
  memset(bt, 0, sizeof(*bt));

  bt->bmem = 0;
  bt->head.right = &bt->z;
  bt->z.left = &bt->z; bt->z.right = &bt->z;

  return(bt);
}


void bt_del_subtree(BT *bt, BTN *x)
{
  if (x->left != &bt->z) bt_del_subtree(bt, x->left);
  if (x->right != &bt->z) bt_del_subtree(bt, x->right);

  if (bt->bmem) bmem_block_del(bt->bmem, x);
  else free(x);
}


void bt_del(BT *bt)
{
  if (bt->head.right != &bt->z) bt_del_subtree(bt, bt->head.right);
  free(bt);
}
