/* Copyright (C) 1999-2000 Hans Petter K. Jansson
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * You can contact the library's author by sending e-mail to <hpj@styx.net>.
 */

#include "config.h"
#include "bt.h"
#include <stdlib.h>


#ifdef __GNUC__
__inline
#endif
void *bt_find(BT *bt, void *key)
{
  BTN *x;
  
  x = bt->head.right;
  bt->z.key = key;
  
  for (;;)
  {
    if (key < x->key)
    {
      x = x->left;
      continue;
    }
    
    if (key > x->key)
    {
      x = x->right;
      continue;
    }
    
    break;
  }

  return(x->data);
}

#ifdef __GNUC__
__inline
#endif
void *bt_find_first_in_range(BT *bt, void *min, void *max)
{
  BTN *x;

  x = bt->head.right;
  bt->z.key = min;

  for (;;)
  {
    if (x->key < min)
    {
      x = x->right;
      continue;
    }
    
    if (x->key > max)
    {
      x = x->left;
      continue;
    }

    break;
  }

  return(x->data);
}


int bt_for_subtree(BT *bt, BTN *btn, int (*func)(void *, void *))
{
  int ret, temp;

  if (btn == &bt->z) return(0);

  temp = func(btn->key, btn->data);
  if (temp < 0) return(temp);
  ret = temp;

  temp = bt_for_subtree(bt, btn->left, func);
  if (temp < 0) return(temp);
  ret += temp;

  temp = bt_for_subtree(bt, btn->right, func);
  if (temp < 0) return(temp);
  ret += temp;
  
  return(ret);
}


/* func() takes two arguments: key and data.
 * Its return value: 0 means go on, +N means add N to total count,
 * -N means stop. */

int bt_for_all(BT *bt, int (*func)(void *, void *))
{
  return(bt_for_subtree(bt, bt->head.right, func));
}
