/* Copyright (C) 1999-2000 Hans Petter K. Jansson
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * You can contact the library's author by sending e-mail to <hpj@styx.net>.
 */

#include "config.h"
#include "nvtp.h"
#include "sock.h"
#include <stdlib.h>


NVTP *_nvtp_alloc()
{
  NVTP *nvtp;
  
  nvtp = malloc(sizeof(*nvtp));
  memset(nvtp, 0, sizeof(*nvtp));

  nvtp->queue_in = tt_new();
  nvtp->queue_out = tt_new();

  return(nvtp);
}


NVTP *nvtp_new()
{
  NVTP *nvtp;
  SOCK *s;
  
  s = sock_new();
  if (!s) return(0);
  
  nvtp = _nvtp_alloc();
  nvtp->s = s;
  return(nvtp);
}


NVTP *nvtp_accepting_new(int port)
{
  NVTP *nvtp;
  SOCK *s;
  
  s = sock_accepting_new(port);
  if (!s) return(0);
  
  nvtp = _nvtp_alloc();
  nvtp->s = s;
  return(nvtp);
}


NVTP *nvtp_pipe_new()
{
  NVTP *nvtp;
  SOCK *s;
  
  s = sock_pipe_new();
  if (!s) return(0);

  nvtp = _nvtp_alloc();
  nvtp->s = s;
  return(nvtp);
}


NVTP *nvtp_new_with_port(int port)
{
  NVTP *nvtp;
  SOCK *s;
  
  s = sock_new_with_port(port);
  if (!s) return(0);
  
  nvtp = _nvtp_alloc();
  nvtp->s = s;
  return(nvtp);
}


NVTP *nvtp_new_with_sock(SOCK *s)
{
  NVTP *nvtp;

  nvtp = _nvtp_alloc();
  nvtp->s = s;
  return(nvtp);
}


void nvtp_del(NVTP *nvtp)
{
  sock_del(nvtp->s);
  if (nvtp->root_in) tt_del(nvtp->root_in);  /* NOTE: Set to zero when queued */
  if (nvtp->root_out) tt_del(nvtp->root_out);  /* NOTE: Set to zero when sent */
  tt_del(nvtp->queue_in);
  tt_del(nvtp->queue_out);
  free(nvtp);
}
