/* Copyright (C) 1999-2000 Hans Petter K. Jansson
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * You can contact the library's author by sending e-mail to <hpj@styx.net>.
 */

#include "config.h"
#include "nvtp.h"
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>


void nvtp_flush(NVTP *nvtp)
{
  while (nvtp_queue_out_len(nvtp))
  {
    nvtp_push(nvtp);
    sock_flush(nvtp_get_sock(nvtp));
  }
  
  sock_flush(nvtp_get_sock(nvtp));
}


void nvtp_grab(NVTP *nvtp)
{
  fd_set fd_read, fd_write;

  while (!nvtp_queue_in_len(nvtp))
  {
    nvtp_pull(nvtp);
    FD_ZERO(&fd_read);
    FD_ZERO(&fd_write);
    FD_SET(nvtp_get_fd_read(nvtp), &fd_read);
    FD_SET(nvtp_get_fd_write(nvtp), &fd_write);

    while (!select(nvtp_get_fd_read(nvtp) > nvtp_get_fd_write(nvtp) ?
                   nvtp_get_fd_read(nvtp) + 1 : nvtp_get_fd_write(nvtp) + 1,
                   &fd_read, &fd_write, 0, 0)) ;
  }
}


/* --- */


void nvtp_send(NVTP *nvtp, char *str)
{
  nvtp_enqueue(nvtp, str);
  nvtp_flush(nvtp);
}


char *nvtp_recv(NVTP *nvtp)
{
  nvtp_grab(nvtp);
  if (!nvtp_queue_in_len(nvtp)) return(0);

  return(nvtp_dequeue(nvtp));
}
