/* Copyright (C) 1999-2000 Hans Petter K. Jansson
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * You can contact the library's author by sending e-mail to <hpj@styx.net>.
 */

#include "config.h"
#include "ph.h"
#include <stdlib.h>


#if (SIZEOF_VOID_P == 4)
# define PH_HASH_FROM_PTR(p) (((psize) (p) >> 2) & (PH_BUCKETS - 1))
#else
# define PH_HASH_FROM_PTR(p) (((psize) (p) >> 3) & (PH_BUCKETS - 1))
#endif


void ph_insert(PH *ph, void *key, void *data)
{
  unsigned int i;
  struct ph_elem *phe;
  
  i = PH_HASH_FROM_PTR(key);

  if (ph->bucket[i])
  {
    for (phe = ph->bucket[i]; ; )
    {
      if (phe->key == key)
      {
        phe->data = data;
        return;
      }
      
      if (phe->next) phe = phe->next;
      else break;
    }
    
    phe->next = malloc(sizeof(*phe));
    phe = phe->next;
  }
  else
  {
    ph->bucket[i] = malloc(sizeof(*phe));
    phe = ph->bucket[i];
  }

  phe->next = 0;
  phe->key = key;
  phe->data = data;
}


void ph_remove(PH *ph, void *key)
{
  unsigned int i;
  struct ph_elem *phe, *phe_prev;

  i = PH_HASH_FROM_PTR(key);

  for (phe = ph->bucket[i], phe_prev = 0; phe; phe = phe->next)
  {
    if (phe->key == key)
    {
      if (!phe_prev) ph->bucket[i] = phe->next;
      else phe_prev->next = phe->next;
      free(phe);
      return;
    }
    
    phe_prev = phe;
  }
}


void *ph_get(PH *ph, void *key)
{
  unsigned int i;
  struct ph_elem *phe;

  i = PH_HASH_FROM_PTR(key);

  for (phe = ph->bucket[i]; phe; phe = phe->next)
    if (phe->key == key) return(phe->data);
  
  return(0);
}
