/* Copyright (C) 1999-2000 Hans Petter K. Jansson
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * You can contact the library's author by sending e-mail to <hpj@styx.net>.
 */

#include "config.h"
#include "log.h"
#include "sock.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>


SOCK *proc_exec_sock(char *command, char *args[], char *env[])
{
  SOCK *s;
  int pid;

  s = sock_pipe_new();

  /* Duplicate process */
  if ((pid = fork()) == 0)
  {
    /* Redirect standard input and output */
    sock_pipe_child(s);
    sock_make_stdio(s);

    /* Do it */
    execve(command, args, 0 /* env ? env : *environ */);
    log_put_opt(LOG_CRIT, 0, "Failed to run command.");
    exit(120);
  }

  /* Close child descriptors and return  */
  else if (pid > 0)
  {
    sock_pipe_parent(s);
    return(s);
  }

  /* Unable to start child */
  sock_del(s);
  return(0);
}
