/* Copyright (C) 1999-2000 Hans Petter K. Jansson
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * You can contact the library's author by sending e-mail to <hpj@styx.net>.
 */

#include "config.h"
#include "tt.h"
#include "proxy.h"

#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#ifdef TIME_WITH_SYS_TIME
# include <time.h>
#endif


/* ------ *
 * Adding *
 * ------ */


void proxy_add_timer(PROXY *p, char *name, struct timeval *interval,
                     int recurrent, void *priv, int (*timeout)(char *, void *))
{
  TT *tt0, *tt1;
  struct timeval tv;

  /* Don't set changed flag, as this only affects sockets */

  gettimeofday(&tv, 0);

  tt0 = tt_find_first_child(p->timers, name, strlen(name));
  if (tt0) tt_del(tt0);

  tt0 = tt_new_with_parent_and_data(p->timers, name, strlen(name));

  tt1 = tt_new_with_parent_and_data(tt0, "priv", 4);
  tt_new_with_parent_and_data(tt1, &priv, sizeof(priv));

  tt1 = tt_new_with_parent_and_data(tt0, "interval", 8);
  tt_new_with_parent_and_data(tt1, interval, sizeof(*interval));

  if (recurrent) tt_new_with_parent_and_data(tt0, "recurrent", 9);

  tt1 = tt_new_with_parent_and_data(tt0, "last", 4);
  tt_new_with_parent_and_data(tt1, &tv, sizeof(tv));
  
  tt1 = tt_new_with_parent_and_data(tt0, "timeout", 7);
  tt_new_with_parent_and_data(tt1, &timeout, sizeof(timeout));
}


/* -------- *
 * Removing *
 * -------- */


void proxy_del_timer(PROXY *p, char *name)
{
  TT *tt0;
  
  /* Don't set changed flag, as this only affects sockets */

  tt0 = tt_find_first_child(p->timers, name, strlen(name));
  if (tt0) tt_del(tt0);
}


/* ------------------ *
 * Special interfaces *
 * ------------------ */


void proxy_reset_timer(PROXY *p, char *name)
{
  TT *tt0;
  struct timeval tv;
  
  tt0 = tt_find_first_child(p->timers, name, strlen(name));
  if (!tt0) return;
  
  gettimeofday(&tv, 0);

  tt0 = tt_find_first_child(tt0, "last", 4);
  tt_del(tt_get_first_child(tt0));
  tt_new_with_parent_and_data(tt0, &tv, sizeof(tv));
}


/* ------- *
 * Calling *
 * ------- */


void proxy_call_timer(PROXY *p, TT *timer)
{
  TT *tt0;
  int (*timeout)(char *, void *);
  void *priv;
  char *timer_name_str;
  struct timeval tv;

  tt0 = tt_get_first_child(tt_find_first_child(timer, "timeout", 7));
  if (!tt0) return;  /* Would be an internal error */
  tt_data_get_bytes(tt0, &timeout, 0, sizeof(timeout));

  tt0 = tt_get_first_child(tt_find_first_child(timer, "priv", 4));
  tt_data_get_bytes(tt0, &priv, 0, sizeof(priv));

  gettimeofday(&tv, 0);
  tt0 = tt_find_first_child(timer, "last", 4);
  tt_del(tt_get_first_child(tt0));
  tt_new_with_parent_and_data(tt0, &tv, sizeof(tv));

  timer_name_str = tt_data_get_str(timer);
  if (!tt_find_first_child(timer, "recurrent", 9)) tt_del(timer);
  
  timeout(timer_name_str, priv);
  free(timer_name_str);
}
