/* Copyright (C) 1999-2000 Hans Petter K. Jansson
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * You can contact the library's author by sending e-mail to <hpj@styx.net>.
 */

#include "config.h"
#include "nvtp.h"
#include "smtp.h"
#include "sock.h"
#include <stdlib.h>


SMTP *_smtp_alloc()
{
  SMTP *smtp;
  
  smtp = malloc(sizeof(*smtp));
  memset(smtp, 0, sizeof(*smtp));

  smtp->queue_in = tt_new();
  smtp->queue_out = tt_new();

  return(smtp);
}


SMTP *smtp_new()
{
  SMTP *smtp;
  NVTP *nvtp;
  
  nvtp = nvtp_new();
  if (!nvtp) return(0);
  
  smtp = _smtp_alloc();
  smtp->nvtp = nvtp;
  return(smtp);
}


SMTP *smtp_accepting_new(int port)
{
  SMTP *smtp;
  NVTP *nvtp;
  
  nvtp = nvtp_accepting_new(port);
  if (!nvtp) return(0);

  smtp = _smtp_alloc();
  smtp->nvtp = nvtp;
  return(smtp);
}


SMTP *smtp_pipe_new()
{
  SMTP *smtp;
  NVTP *nvtp;
  
  nvtp = nvtp_pipe_new();
  if (!nvtp) return(0);

  smtp = _smtp_alloc();
  smtp->nvtp = nvtp;
  return(smtp);
}


SMTP *smtp_new_with_port(int port)
{
  SMTP *smtp;
  NVTP *nvtp;
  
  nvtp = nvtp_new_with_port(port);
  if (!nvtp) return(0);
  
  smtp = _smtp_alloc();
  smtp->nvtp = nvtp;
  return(smtp);
}


SMTP *smtp_new_with_sock(SOCK *s)
{
  SMTP *smtp;

  smtp = _smtp_alloc();
  smtp->nvtp = nvtp_new_with_sock(s);
  return(smtp);
}


SMTP *smtp_new_with_nvtp(NVTP *nvtp)
{
  SMTP *smtp;

  smtp = _smtp_alloc();
  smtp->nvtp = nvtp;
  return(smtp);
}


void smtp_del(SMTP *smtp)
{
  nvtp_del(smtp->nvtp);
  if (smtp->root_in) tt_del(smtp->root_in);  /* NOTE: Set to zero when queued */
  if (smtp->root_out) tt_del(smtp->root_out);  /* NOTE: Set to zero when sent */
  tt_del(smtp->queue_in);
  tt_del(smtp->queue_out);
  free(smtp);
}
