/* Copyright (C) 1999-2000 Hans Petter K. Jansson
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * You can contact the library's author by sending e-mail to <hpj@styx.net>.
 */


#include "config.h"
#include "bmem.h"
#include "test.h"
#include <stdlib.h>

#define CONCURRENT_BLOCKS 8192
#define MALLOC_ITERATIONS 250


int test_malloc_speed()
{
  void *p[CONCURRENT_BLOCKS];
  int i, j;
  double r1;
  
  bm_start();

  for (i = MALLOC_ITERATIONS; i; i--)
  {
    for (j = 0; j < CONCURRENT_BLOCKS; j++) p[j] = malloc(256);
    for (j = 0; j < CONCURRENT_BLOCKS; j++) free(p[j]);
  }

  bm_end();

  r1 = (double) (((double) CONCURRENT_BLOCKS * (double) MALLOC_ITERATIONS * 1000000.0) / (double) bm_diff_get_usec());
  return((int) r1);
}


int test_bmem_speed()
{
  BMEM *bm;
  void *p[CONCURRENT_BLOCKS];
  int i, j;
  double r1;

  bm = bmem_new(256);
  
  bm_start();

  for (i = MALLOC_ITERATIONS * 2; i; i--)
  {
    for (j = 0; j < CONCURRENT_BLOCKS-1; j++) p[j] = bmem_block_new(bm);
    for (j = 0; j < CONCURRENT_BLOCKS-1; j++) bmem_block_del(bm, p[j]);
  }

  bm_end();

  bmem_del(bm);
  
  r1 = (double) (((double) CONCURRENT_BLOCKS * (double) MALLOC_ITERATIONS * 2.0 * 1000000.0) / (double) bm_diff_get_usec());
  return((int) r1);
}
