/* Copyright (C) 1999-2000 Hans Petter K. Jansson
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * You can contact the library's author by sending e-mail to <hpj@styx.net>.
 */

#include "config.h"
#include "test.h"
#include "bt.h"
#include "flux.h"
#include <stdlib.h>
#include <unistd.h>

#define TEST_VALUES 8192


BT *bt;


int test_bt_integrity()
{
  unsigned int i;
  unsigned long val[TEST_VALUES];
  int r = 0;

  bt = bt_new();
  
  for (i = 0; i < TEST_VALUES; i++)
  {
    val[i] = rand();
    bt_insert(bt, (void *) val[i], (void *) val[i]);
  }
  
  for (i = 0; i < TEST_VALUES; i++)
  {
    if ((unsigned long) bt_find(bt, (void *) val[i]) != val[i]) r = -1;
    if ((unsigned long) bt_find_first_in_range(bt, (void *) val[i], (void *) val[i]) != val[i]) r = -1;
    bt_remove(bt, (void *) val[i]);
  }
  
  if (!bt_is_empty(bt)) return(-1);
  
  for (i = 0; i < TEST_VALUES; i++)
    if (val[i] && (unsigned long) bt_find(bt, (void *) val[i]) == val[i]) r = -1;
  
  bt_del(bt);
  return(r);
}


int test_bt_insert_speed()
{
  unsigned int i;
  double r1;

  bt = bt_new();
  
  bm_start();
  for (i = 0; i < 500000; i++) bt_insert(bt, (void *) ((unsigned long) rand() % 500000), (void *) ((unsigned long) 4));
  bm_end();

  r1 = (double) (500000.0 * 1000000.0) / (double) bm_diff_get_usec();
  return((int) r1);
}


int test_bt_search_speed()
{
  unsigned int i;
  double r1;

  bm_start();
  for (i = 0; i < 500000; i++) bt_find(bt, (void *) ((unsigned long) rand() % 500000));
  bm_end();

  bt_del(bt);

  r1 = (double) (500000.0 * 1000000.0) / (double) bm_diff_get_usec();
  return((int) r1);
}
