/* Copyright (C) 1999-2000 Hans Petter K. Jansson
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * You can contact the library's author by sending e-mail to <hpj@styx.net>.
 */


#include "config.h"
#include "random.h"
#include <stdio.h>
#include <math.h>

/*
 *  Ueli M Maurer's "Universal Statistical Test for Random Bit Generators",
 *  using L = 8.
 *
 *  Expected value for blocksize = 8 is 7.1836656.
 */

#define L 8
#define V (1 << L)
#define Q (10 * V)
#define K (100 * Q)
#define MAXSAMP (Q + K)

static double maurer_test(unsigned char *p, unsigned int len)
{
  int i,j;
  int b;
  int table[V];
  double sum = 0.0;
  int iproduct = 1;
  
  extern double log(/* double x */);

  for (i = 0; i < V; i++) table[i] = 0;
  for (i = 0; len && i < Q; i++)
  {
    b = *p++; len--;
    table[b] = i;
  }

  sum = 0.0;
  iproduct = 1;
    
  for (i = Q; len && i < Q + K; i++)
  {
    j = i;
    b = *p++; len--;
    
    if (table[b] > j) j += K;
    sum += log((double) (j - table[b]));
    
    table[b] = i;
  }
  
  sum = (sum / ((double) (i - Q))) / log(2.0);
  
  return(sum);
}


int test_random_low()
{
  unsigned char buf[8192];

  rand_block(buf, 8192, 0);
  if (maurer_test(buf, 8192) > 6.0) return(0);
  return(-1);
}


int test_random_medium()
{
  unsigned char buf[8192];

  rand_block(buf, 8192, 1);
  if (maurer_test(buf, 8192) > 6.8) return(0);
  return(-1);
}


int test_random_high()
{
  unsigned char buf[20];
  
  rand_block(buf, 20, 2);
  if (maurer_test(buf, 8192) > 7.0) return(0);
  return(-1);
}
